import { apply, divide, filter, path } from 'ramda';

import { printError } from '../../lib';
import { ASPECT_RATIO_FORMAT } from '../../lib/customPropTypes';

/**
 * Relative error threshold for images
 * @returns {number}
 */
const THRESHOLD = 0.05;

/**
 * Filters image sizes based on an aspect ratio
 * Pass `media.media_details.sizes` along with an aspect ratio in order to
 * obtain all sizes that are close to that aspect ratio.
 * If an aspect ratio is not passed then the sizes will be filtered to include
 * sizes matching the original image aspect ratio.
 * @param {Object} mediaSizes - List of image sizes to compare.
 * @param {string} aspectRatio - Ratio to compare by (or `false` for original).
 * @returns {Object} Image sizes that fit into the range of the aspect ratio
 */
const filterSizesByAspectRatio = (mediaSizes = {}, aspectRatio) => {
  if (!aspectRatio) {
    const width = path(['full', 'width'], mediaSizes);
    const height = path(['full', 'height'], mediaSizes);
    aspectRatio = `${width}:${height}`;
  }

  if (!ASPECT_RATIO_FORMAT.test(aspectRatio)) {
    printError('Invalid aspect ratio provided.');
    return {};
  }

  const aspectRatioNumeric = apply(divide, aspectRatio.split(':'));

  return filter(({ height, width }) => {
    const absoluteError = Math.abs(aspectRatioNumeric - (width / height));
    const relativeError = absoluteError / aspectRatioNumeric;

    return relativeError < THRESHOLD;
  }, mediaSizes);
};

export default filterSizesByAspectRatio;
