import { shallow } from 'enzyme';
import React from 'react';
import { assoc, forEach } from 'ramda';

import { MediaVideo } from './MediaVideo';
import DefaultItem from '../Item/Item';
import { Site } from '../../../tests/storyshare';

jest.mock('../../../public/lib/jwplayer/jwplayer', () => null);
jest.useFakeTimers();

describe('MediaVideo', () => {
  const VIDEO_FIXTURE = {
    id: 23,
    endpoints: {
      hls: 'http://examplehls.com',
      preview_thumbnails: 'http://examplepreview.com',
      offline: 'http://offline.com',
      subtitles: { en: 'test' },
    },
    playback: {},
  };

  const PLAYLIST_FIXTURE = [{
    mediaid: 23,
    tracks: [
      { 'file': 'http://examplepreview.com', 'kind': 'thumbnails' },
      { 'file': 'test', 'kind': 'captions', 'label': 'Default subtitles' },
    ],
    sources: [{
      'file': 'http://examplehls.com', 'type': 'hls',
    }],
  }];

  const IMAGE_FIXTURE = { media_details: { sizes: {
    '640x360': { source_url: 'path/to/640x360', width: 640, height: 360 },
  } } };

  let wrapper;

  const site = new Site();

  beforeEach(() => {
    jest.spyOn(site, 'objectToResponse');
    mockJwplayer.on.mockClear();
    mockJwplayer.getState.mockClear();
    site.objectToResponse.mockClear();
    window.__webpack_public_path__ = '/test/path/';
    wrapper = shallow(
      <MediaVideo id={ 23 } posterWidth={ 640 } site={ site } />
    );
  });

  const getVideoItem = () => {
    site.objectToResponse.mockReturnValue({
      offline: jest.fn().mockReturnValueOnce(VIDEO_FIXTURE),
    });
    const children = wrapper.find(DefaultItem).prop('children');
    return shallow(<div>{ children(VIDEO_FIXTURE) }</div>);
  };

  const updateVideoFixture = (newVideoFixture) =>
    site.objectToResponse.mockReturnValueOnce({
      offline: jest.fn().mockReturnValueOnce(newVideoFixture),
    });

  const getImageItem = () => {
    site.objectToResponse.mockReturnValue({
      offline: jest.fn().mockReturnValueOnce(IMAGE_FIXTURE),
    });
    const children = getVideoItem().find(DefaultItem).prop('children');
    return shallow(<div>{ children(IMAGE_FIXTURE) }</div>);
  };

  const mockJwplayer = {
    on: jest.fn(),
    getState: jest.fn(),
  };

  const mountPlayer = () =>
    getVideoItem().find('JwPlayer').simulate('playerMounted', mockJwplayer);

  const getJwplayerEventHandlers = () => {
    const eventHandlers = {};

    mockJwplayer.on.mockImplementation((event, handler) => {
      eventHandlers[event] = handler;
    });

    mountPlayer();

    return eventHandlers;
  };

  const sendTimeTrackEvent = (time) => {
    site.trackEvent.mockClear();

    const eventHandlers = getJwplayerEventHandlers();

    expect(eventHandlers).toHaveProperty('time');

    eventHandlers.time({ duration: 100, currentTime: time });
  };

  it('renders', () => {
    expect(wrapper).toExist();
  });

  it('renders an Item to retrieve the video media', () => {
    const videoItem = wrapper.find(DefaultItem);

    expect(videoItem).toExist();
    expect(videoItem).toHaveProp('id', 23);
    expect(videoItem).toHaveProp('typeSlug', 'media');
  });

  it('calls offline when video is requested', () => {
    const videoResponse = {
      offline: jest.fn().mockReturnValue(VIDEO_FIXTURE),
    };
    site.objectToResponse.mockReturnValueOnce(videoResponse);

    getVideoItem();

    expect(videoResponse.offline).toHaveBeenCalled();
    expect(videoResponse.offline()).toEqual(VIDEO_FIXTURE);
  });

  it('renders jwplayer', () => {
    expect(getVideoItem().find('JwPlayer')).toExist();
  });

  it('passes aspectratio to jwplayerconfig if provided', () => {
    const newVideoFixture = assoc('playback',
      { aspect_ratio: '4:3' },
      VIDEO_FIXTURE
    );

    updateVideoFixture(newVideoFixture);

    const videoAspectRatio = getVideoItem()
      .find('JwPlayer')
      .prop('jwPlayerConfig')
      .aspectratio;

    expect(videoAspectRatio).toBe('4:3');
  });

  it('passes props to jwplayer with default aspectratio included', () => {
    const jwPlayerConfig = {
      aspectratio: '16:9',
      playlist: PLAYLIST_FIXTURE,
    };

    expect(getVideoItem().find('JwPlayer'))
      .toHaveProp('jwPlayerConfig', jwPlayerConfig);
  });

  it('passes extra props to jwplayer', () => {
    wrapper.setProps({ testProp: 'propValue' });

    expect(getVideoItem().find('JwPlayer')).toHaveProp('testProp', 'propValue');
  });

  it('renders image with posterId if set', () => {
    wrapper.setProps({ posterId: 15 });

    const imageItem = getVideoItem().find(DefaultItem);

    expect(imageItem).toHaveProp('id', 15);
  });

  it('renders an item to retrieve image media if featured_media exist', () => {
    const newVideoFixture = assoc('featured_media', 42, VIDEO_FIXTURE);

    updateVideoFixture(newVideoFixture);

    const imageItem = getVideoItem().find(DefaultItem);

    expect(imageItem).toExist();
    expect(imageItem).toHaveProp('id', 42);
    expect(imageItem).toHaveProp('typeSlug', 'media');
  });

  it('calls imageurl to jwplayer if featured_media exist', () => {
    const newVideoFixture = assoc('featured_media', 42, VIDEO_FIXTURE);

    updateVideoFixture(newVideoFixture);

    const spy = jest.spyOn(wrapper.instance(), 'getImageUrlByWidth');
    getImageItem();

    expect(spy).toHaveBeenCalled();
  });

  it('binds an event handler', () => {
    mountPlayer();
    expect(mockJwplayer.on).toHaveBeenCalled();
  });

  it('calls onPlayerMounted prop if it is defined', () => {
    const onPlayerMounted = jest.fn();
    wrapper.setProps({ onPlayerMounted });

    mountPlayer();

    expect(onPlayerMounted).toHaveBeenCalled();
  });

  const jwplayerEvents = [
    { event: 'play', action: 'play' },
    { event: 'pause', action: 'pause' },
    { event: 'complete', action: 'ended' },
  ];

  forEach(({ action, event }) => {
    it(`calls jwplayer ${event} and sends track event info`, () => {
      jest.spyOn(site, 'trackEvent');
      site.trackEvent.mockClear();

      const eventHandlers = getJwplayerEventHandlers();

      expect(eventHandlers).toHaveProperty(event);

      eventHandlers[event]();

      expect(site.trackEvent).toHaveBeenCalled();
      expect(site.trackEvent).toHaveBeenCalledWith(
        site.ANALYTICS_TYPE.MEDIA,
        { 'action': action, 'objectRoute': 'media/23' }
      );
    });
  }, jwplayerEvents);

  const eventTimers = [
    { time: 10 },
    { time: 25 },
    { time: 50 },
    { time: 75 },
    { time: 90 },
  ];

  forEach(({ time }) => {
    it(`calls jwplayer time and sends track event info when it is ${time}%`,
      () => {
        sendTimeTrackEvent(time);

        expect(site.trackEvent).toHaveBeenCalled();
        expect(site.trackEvent).toHaveBeenCalledWith(
          site.ANALYTICS_TYPE.MEDIA,
          {
            'action': `${time}%`,
            'objectRoute': 'media/23',
            interactive: false,
          }
        );
      });
  }, eventTimers);

  it('does not send track events while seeking', () => {
    jest.spyOn(site, 'trackEvent');
    site.trackEvent.mockClear();

    const eventHandlers = getJwplayerEventHandlers();
    eventHandlers.seek();

    eventHandlers.time({ duration: 100, currentTime: 90 });

    expect(site.trackEvent).not.toHaveBeenCalled();
  });

  it('does not send track events just after seeking', () => {
    jest.spyOn(site, 'trackEvent');
    site.trackEvent.mockClear();

    const eventHandlers = getJwplayerEventHandlers();
    eventHandlers.seek();
    eventHandlers.seeked();

    eventHandlers.time({ duration: 100, currentTime: 90 });

    expect(site.trackEvent).not.toHaveBeenCalled();
  });

  it('sends tracking event only after post-seeked delay', () => {
    jest.spyOn(site, 'trackEvent');
    site.trackEvent.mockClear();

    const eventHandlers = getJwplayerEventHandlers();
    eventHandlers.seek();
    eventHandlers.seeked();

    jest.advanceTimersByTime(500);

    eventHandlers.time({ duration: 100, currentTime: 75 });

    expect(site.trackEvent).toHaveBeenCalledTimes(1);
    expect(site.trackEvent).toHaveBeenCalledWith(
      site.ANALYTICS_TYPE.MEDIA,
      { 'action': '75%', 'objectRoute': 'media/23', interactive: false }
    );
  });

  it('sends additional track data if exist after post-seeked delay', () => {
    wrapper.setProps({ additionalTrackingProperties: { example: 'test' } });
    jest.spyOn(site, 'trackEvent');
    site.trackEvent.mockClear();

    const eventHandlers = getJwplayerEventHandlers();
    eventHandlers.seek();
    eventHandlers.seeked();

    jest.advanceTimersByTime(500);

    eventHandlers.time({ duration: 100, currentTime: 75 });

    expect(site.trackEvent).toHaveBeenCalledTimes(1);
    expect(site.trackEvent).toHaveBeenCalledWith(
      site.ANALYTICS_TYPE.MEDIA,
      {
        'action': '75%',
        'objectRoute': 'media/23',
        interactive: false,
        example: 'test',
      }
    );
  });

  forEach(({ time }) => {
    it(`sends additional tracking event if passed in when it is ${time}%`,
      () => {
        wrapper.setProps({ additionalTrackingProperties: { example: 'test' } });
        sendTimeTrackEvent(time);

        expect(site.trackEvent).toHaveBeenCalled();
        expect(site.trackEvent).toHaveBeenCalledWith(
          site.ANALYTICS_TYPE.MEDIA,
          {
            'action': `${time}%`,
            'objectRoute': 'media/23',
            interactive: false,
            example: 'test',
          }
        );
      });
  }, eventTimers);

  forEach(({ action, event }) => {
    it(`sends additional tracking event when jwplayer ${event} occurs`, () => {
      wrapper.setProps({ additionalTrackingProperties: { example: 'test' } });
      jest.spyOn(site, 'trackEvent');
      site.trackEvent.mockClear();

      const eventHandlers = getJwplayerEventHandlers();

      expect(eventHandlers).toHaveProperty(event);

      eventHandlers[event]();

      expect(site.trackEvent).toHaveBeenCalled();
      expect(site.trackEvent).toHaveBeenCalledWith(
        site.ANALYTICS_TYPE.MEDIA,
        { 'action': action, 'objectRoute': 'media/23', example: 'test' }
      );
    });
  }, jwplayerEvents);

  it('does not update props if additionalTrackingProperties changes', () => {
    wrapper.setProps({ additionalTrackingProperties: { sample: 'old' } });
    const allProps = wrapper.instance().props;

    const shouldUpdate = wrapper.instance().shouldComponentUpdate({
      ...allProps, additionalTrackingProperties: { sample: 'newsample' } }
    );

    expect(shouldUpdate).toBe(false);
  });

  it('update props if other prop changes changes', () => {
    wrapper.setProps({ posterId: 42 });
    const allProps = wrapper.instance().props;

    const shouldUpdate = wrapper.instance().shouldComponentUpdate({
      ...allProps, posterId: 45 }
    );

    expect(shouldUpdate).toBe(true);
  });
});
